use std::io::Read;
use std::process::{Command, Stdio};

use anyhow::{Context, Result};
use chrono::{DateTime, Local, NaiveDateTime, Utc};
use serde::{Deserialize, Deserializer};

use super::Entry;

pub fn from_timew_export() -> Result<Vec<Entry>> {
    let mut child = Command::new("timew")
        .arg("export")
        .stdout(Stdio::piped())
        .spawn()
        .with_context(|| "running 'timew export'")?;

    let stdout = child
        .stdout
        .take()
        .with_context(|| "take 'timew export' output")?;

    let credit = from_reader(stdout)?;

    child.wait().with_context(|| "waiting on 'timew export'")?;

    Ok(credit)
}

pub fn from_reader<R>(r: R) -> Result<Vec<Entry>>
where
    R: Read,
{
    let timew: Vec<Data> = serde_json::from_reader(r)?;
    let credit = timew
        .into_iter()
        .filter_map(|timew| {
            timew.end.map(|end| {
                let duration = end - timew.start;

                Entry {
                    date: timew.start.date_naive(),
                    seconds: duration.num_seconds(),
                }
            })
        })
        .collect();

    Ok(credit)
}

// ----------------------------------------------------------------------------
// serde helper
// ----------------------------------------------------------------------------

#[derive(Debug, Deserialize)]
pub struct Data {
    #[allow(dead_code)]
    id: usize,

    #[serde(deserialize_with = "timew_to_local")]
    start: DateTime<Local>,

    #[serde(default)]
    #[serde(deserialize_with = "timew_end_to_local")]
    end: Option<DateTime<Local>>,

    #[allow(dead_code)]
    #[serde(default)]
    tags: Vec<String>,

    #[allow(dead_code)]
    annotation: Option<String>,
}

fn timew_to_local<'de, D>(deserializer: D) -> Result<DateTime<Local>, D::Error>
where
    D: Deserializer<'de>,
{
    let s = String::deserialize(deserializer)?;
    let dt = NaiveDateTime::parse_from_str(&s, "%Y%m%dT%H%M%SZ")
        .map_err(serde::de::Error::custom)?;

    let dt = DateTime::<Utc>::from_naive_utc_and_offset(dt, Utc);
    let dt = DateTime::from(dt);

    Ok(dt)
}

fn timew_end_to_local<'de, D>(
    deserializer: D,
) -> Result<Option<DateTime<Local>>, D::Error>
where
    D: Deserializer<'de>,
{
    timew_to_local(deserializer).map(Some)
}
